@@ -15,11 +15,10 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::builder::{ArrayBuilder, BufferBuilder, UInt8BufferBuilder};
+use crate::builder::ArrayBuilder;
 use crate::types::{ByteArrayType, GenericBinaryType, GenericStringType};
 use crate::{Array, ArrayRef, GenericByteArray, OffsetSizeTrait};
-use arrow_buffer::NullBufferBuilder;
-use arrow_buffer::{ArrowNativeType, Buffer, MutableBuffer};
+use arrow_buffer::{ArrowNativeType, Buffer, MutableBuffer, NullBufferBuilder, ScalarBuffer};
 use arrow_data::ArrayDataBuilder;
 use std::any::Any;
 use std::sync::Arc;
@@ -29,8 +28,8 @@ use std::sync::Arc;
 /// For building strings, see docs on [`GenericStringBuilder`].
 /// For building binary, see docs on [`GenericBinaryBuilder`].
 pub struct GenericByteBuilder<T: ByteArrayType> {
-    value_builder: UInt8BufferBuilder,
-    offsets_builder: BufferBuilder<T::Offset>,
+    value_builder: Vec<u8>,
+    offsets_builder: Vec<T::Offset>,
     null_buffer_builder: NullBufferBuilder,
 }
 
@@ -47,10 +46,10 @@ impl<T: ByteArrayType> GenericByteBuilder<T> {
     /// - `data_capacity` is the total number of bytes of data to pre-allocate
     ///   (for all items, not per item).
     pub fn with_capacity(item_capacity: usize, data_capacity: usize) -> Self {
-        let mut offsets_builder = BufferBuilder::<T::Offset>::new(item_capacity + 1);
-        offsets_builder.append(T::Offset::from_usize(0).unwrap());
+        let mut offsets_builder = Vec::with_capacity(item_capacity + 1);
+        offsets_builder.push(T::Offset::from_usize(0).unwrap());
         Self {
-            value_builder: UInt8BufferBuilder::new(data_capacity),
+            value_builder: Vec::with_capacity(data_capacity),
             offsets_builder,
             null_buffer_builder: NullBufferBuilder::new(item_capacity),
         }
@@ -67,8 +66,9 @@ impl<T: ByteArrayType> GenericByteBuilder<T> {
         value_buffer: MutableBuffer,
         null_buffer: Option<MutableBuffer>,
     ) -> Self {
-        let offsets_builder = BufferBuilder::<T::Offset>::new_from_buffer(offsets_buffer);
-        let value_builder = BufferBuilder::<u8>::new_from_buffer(value_buffer);
+        let offsets_builder: Vec<T::Offset> =
+            ScalarBuffer::<T::Offset>::from(offsets_buffer).into();
+        let value_builder: Vec<u8> = ScalarBuffer::<u8>::from(value_buffer).into();
 
         let null_buffer_builder = null_buffer
             .map(|buffer| NullBufferBuilder::new_from_buffer(buffer, offsets_builder.len() - 1))
@@ -103,9 +103,10 @@ impl<T: ByteArrayType> GenericByteBuilder<T> {
     /// [`BinaryArray`]: crate::BinaryArray
     #[inline]
     pub fn append_value(&mut self, value: impl AsRef<T::Native>) {
-        self.value_builder.append_slice(value.as_ref().as_ref());
+        self.value_builder
+            .extend_from_slice(value.as_ref().as_ref());
         self.null_buffer_builder.append(true);
-        self.offsets_builder.append(self.next_offset());
+        self.offsets_builder.push(self.next_offset());
     }
 
     /// Append an `Option` value into the builder.
@@ -126,7 +127,7 @@ impl<T: ByteArrayType> GenericByteBuilder<T> {
     #[inline]
     pub fn append_null(&mut self) {
         self.null_buffer_builder.append(false);
-        self.offsets_builder.append(self.next_offset());
+        self.offsets_builder.push(self.next_offset());
     }
 
     /// Appends `n` `null`s into the builder.
@@ -134,7 +135,8 @@ impl<T: ByteArrayType> GenericByteBuilder<T> {
     pub fn append_nulls(&mut self, n: usize) {
         self.null_buffer_builder.append_n_nulls(n);
         let next_offset = self.next_offset();
-        self.offsets_builder.append_n(n, next_offset);
+        self.offsets_builder
+            .extend(std::iter::repeat_n(next_offset, n));
     }
 
     /// Appends array values and null to this builder as is
@@ -150,7 +152,7 @@ impl<T: ByteArrayType> GenericByteBuilder<T> {
         // If the offsets are contiguous, we can append them directly avoiding the need to align
         // for example, when the first appended array is not sliced (starts at offset 0)
         if self.next_offset() == offsets[0] {
-            self.offsets_builder.append_slice(&offsets[1..]);
+            self.offsets_builder.extend_from_slice(&offsets[1..]);
         } else {
             // Shifting all the offsets
             let shift: T::Offset = self.next_offset() - offsets[0];
@@ -164,11 +166,11 @@ impl<T: ByteArrayType> GenericByteBuilder<T> {
                 intermediate.push(offset + shift)
             }
 
-            self.offsets_builder.append_slice(&intermediate);
+            self.offsets_builder.extend_from_slice(&intermediate);
         }
 
         // Append underlying values, starting from the first offset and ending at the last offset
-        self.value_builder.append_slice(
+        self.value_builder.extend_from_slice(
             &array.values().as_slice()[offsets[0].as_usize()..offsets[array.len()].as_usize()],
         );
 
@@ -184,11 +186,11 @@ impl<T: ByteArrayType> GenericByteBuilder<T> {
         let array_type = T::DATA_TYPE;
         let array_builder = ArrayDataBuilder::new(array_type)
             .len(self.len())
-            .add_buffer(self.offsets_builder.finish())
-            .add_buffer(self.value_builder.finish())
+            .add_buffer(std::mem::take(&mut self.offsets_builder).into())
+            .add_buffer(std::mem::take(&mut self.value_builder).into())
             .nulls(self.null_buffer_builder.finish());
 
-        self.offsets_builder.append(self.next_offset());
+        self.offsets_builder.push(self.next_offset());
         let array_data = unsafe { array_builder.build_unchecked() };
         GenericByteArray::from(array_data)
     }
@@ -340,7 +342,7 @@ pub type GenericStringBuilder<O> = GenericByteBuilder<GenericStringType<O>>;
 
 impl<O: OffsetSizeTrait> std::fmt::Write for GenericStringBuilder<O> {
     fn write_str(&mut self, s: &str) -> std::fmt::Result {
-        self.value_builder.append_slice(s.as_bytes());
+        self.value_builder.extend_from_slice(s.as_bytes());
         Ok(())
     }
 }
@@ -394,7 +396,7 @@ pub type GenericBinaryBuilder<O> = GenericByteBuilder<GenericBinaryType<O>>;
 
 impl<O: OffsetSizeTrait> std::io::Write for GenericBinaryBuilder<O> {
     fn write(&mut self, bs: &[u8]) -> std::io::Result<usize> {
-        self.value_builder.append_slice(bs);
+        self.value_builder.extend_from_slice(bs);
         Ok(bs.len())
     }
 
