@@ -217,14 +217,28 @@ pub(crate) fn split_second(v: i64, base: i64) -> (i64, u32) {
 
 /// converts a `i64` representing a `duration(s)` to [`Duration`]
 #[inline]
+#[deprecated(since = "55.2.0", note = "Use `try_duration_s_to_duration` instead")]
 pub fn duration_s_to_duration(v: i64) -> Duration {
     Duration::try_seconds(v).unwrap()
 }
 
+/// converts a `i64` representing a `duration(s)` to [`Option<Duration>`]
+#[inline]
+pub fn try_duration_s_to_duration(v: i64) -> Option<Duration> {
+    Duration::try_seconds(v)
+}
+
 /// converts a `i64` representing a `duration(ms)` to [`Duration`]
 #[inline]
+#[deprecated(since = "55.2.0", note = "Use `try_duration_ms_to_duration` instead")]
 pub fn duration_ms_to_duration(v: i64) -> Duration {
-    Duration::try_milliseconds(v).unwrap()
+    Duration::try_seconds(v).unwrap()
+}
+
+/// converts a `i64` representing a `duration(ms)` to [`Option<Duration>`]
+#[inline]
+pub fn try_duration_ms_to_duration(v: i64) -> Option<Duration> {
+    Duration::try_milliseconds(v)
 }
 
 /// converts a `i64` representing a `duration(us)` to [`Duration`]
@@ -296,8 +310,8 @@ pub fn as_time<T: ArrowPrimitiveType>(v: i64) -> Option<NaiveTime> {
 pub fn as_duration<T: ArrowPrimitiveType>(v: i64) -> Option<Duration> {
     match T::DATA_TYPE {
         DataType::Duration(unit) => match unit {
-            TimeUnit::Second => Some(duration_s_to_duration(v)),
-            TimeUnit::Millisecond => Some(duration_ms_to_duration(v)),
+            TimeUnit::Second => try_duration_s_to_duration(v),
+            TimeUnit::Millisecond => try_duration_ms_to_duration(v),
             TimeUnit::Microsecond => Some(duration_us_to_duration(v)),
             TimeUnit::Nanosecond => Some(duration_ns_to_duration(v)),
         },
