@@ -17,6 +17,11 @@
 
 //! Functions for converting from one data type to another in [Apache Arrow](https://docs.rs/arrow)
 
+#![doc(
+    html_logo_url = "https://arrow.apache.org/img/arrow-logo_chevrons_black-txt_white-bg.svg",
+    html_favicon_url = "https://arrow.apache.org/img/arrow-logo_chevrons_black-txt_transparent-bg.svg"
+)]
+#![cfg_attr(docsrs, feature(doc_auto_cfg))]
 #![warn(missing_docs)]
 pub mod cast;
 pub use cast::*;
