@@ -299,7 +299,7 @@ pub struct GetXdbcTypeInfoBuilder<'a> {
 
 impl CommandGetXdbcTypeInfo {
     /// Create a builder suitable for constructing a response
-    pub fn into_builder(self, infos: &XdbcTypeInfoData) -> GetXdbcTypeInfoBuilder {
+    pub fn into_builder(self, infos: &XdbcTypeInfoData) -> GetXdbcTypeInfoBuilder<'_> {
         GetXdbcTypeInfoBuilder {
             data_type: self.data_type,
             infos,
