@@ -17,12 +17,9 @@
 
 //! Helper trait [`FlightSqlService`] for implementing a [`FlightService`] that implements FlightSQL.
 
+use std::fmt::{Display, Formatter};
 use std::pin::Pin;
 
-use futures::{stream::Peekable, Stream, StreamExt};
-use prost::Message;
-use tonic::{Request, Response, Status, Streaming};
-
 use super::{
     ActionBeginSavepointRequest, ActionBeginSavepointResult, ActionBeginTransactionRequest,
     ActionBeginTransactionResult, ActionCancelQueryRequest, ActionCancelQueryResult,
@@ -41,6 +38,9 @@ use crate::{
     FlightData, FlightDescriptor, FlightInfo, HandshakeRequest, HandshakeResponse, PutResult,
     SchemaResult, Ticket,
 };
+use futures::{stream::Peekable, Stream, StreamExt};
+use prost::Message;
+use tonic::{Request, Response, Status, Streaming};
 
 pub(crate) static CREATE_PREPARED_STATEMENT: &str = "CreatePreparedStatement";
 pub(crate) static CLOSE_PREPARED_STATEMENT: &str = "ClosePreparedStatement";
@@ -386,6 +386,15 @@ pub trait FlightSqlService: Sync + Send + Sized + 'static {
         )))
     }
 
+    /// Implementors may override to handle do_put errors
+    async fn do_put_error_callback(
+        &self,
+        _request: Request<PeekableFlightDataStream>,
+        error: DoPutError,
+    ) -> Result<Response<<Self as FlightService>::DoPutStream>, Status> {
+        Err(Status::unimplemented(format!("Unhandled Error: {error}")))
+    }
+
     /// Execute an update SQL statement.
     async fn do_put_statement_update(
         &self,
@@ -710,10 +719,21 @@ where
         // we wrap this stream in a `Peekable` one, which allows us to peek at
         // the first message without discarding it.
         let mut request = request.map(PeekableFlightDataStream::new);
-        let cmd = Pin::new(request.get_mut()).peek().await.unwrap().clone()?;
+        let mut stream = Pin::new(request.get_mut());
 
-        let message =
-            Any::decode(&*cmd.flight_descriptor.unwrap().cmd).map_err(decode_error_to_status)?;
+        let peeked_item = stream.peek().await.cloned();
+        let Some(cmd) = peeked_item else {
+            return self
+                .do_put_error_callback(request, DoPutError::MissingCommand)
+                .await;
+        };
+
+        let Some(flight_descriptor) = cmd?.flight_descriptor else {
+            return self
+                .do_put_error_callback(request, DoPutError::MissingFlightDescriptor)
+                .await;
+        };
+        let message = Any::decode(flight_descriptor.cmd).map_err(decode_error_to_status)?;
         match Command::try_from(message).map_err(arrow_error_to_status)? {
             Command::CommandStatementUpdate(command) => {
                 let record_count = self.do_put_statement_update(command, request).await?;
@@ -968,6 +988,26 @@ where
     }
 }
 
+/// Unrecoverable errors associated with `do_put` requests
+pub enum DoPutError {
+    /// The first element in the request stream is missing the command
+    MissingCommand,
+    /// The first element in the request stream is missing the flight descriptor
+    MissingFlightDescriptor,
+}
+impl Display for DoPutError {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            DoPutError::MissingCommand => {
+                write!(f, "Command is missing.")
+            }
+            DoPutError::MissingFlightDescriptor => {
+                write!(f, "Flight descriptor is missing.")
+            }
+        }
+    }
+}
+
 fn decode_error_to_status(err: prost::DecodeError) -> Status {
     Status::invalid_argument(format!("{err:?}"))
 }
