@@ -109,6 +109,11 @@ impl StreamDecoder {
         self
     }
 
+    /// Return the schema if decoded, else None.
+    pub fn schema(&self) -> Option<SchemaRef> {
+        self.schema.as_ref().map(|schema| schema.clone())
+    }
+
     /// Try to read the next [`RecordBatch`] from the provided [`Buffer`]
     ///
     /// [`Buffer::advance`] will be called on `buffer` for any consumed bytes.
@@ -129,6 +134,9 @@ impl StreamDecoder {
     ///             if let Some(x) = decoder.decode(&mut x)? {
     ///                 println!("{x:?}");
     ///             }
+    ///             if let Some(schema) = decoder.schema() {
+    ///                 println!("Schema: {schema:?}");
+    ///             }
     ///         }
     ///     }
     ///     decoder.finish().unwrap();
@@ -326,6 +334,31 @@ mod tests {
         assert_eq!(err, "Ipc error: Unexpected End of Stream");
     }
 
+    #[test]
+    fn test_schema() {
+        let schema = Arc::new(Schema::new(vec![
+            Field::new("int32", DataType::Int32, false),
+            Field::new("int64", DataType::Int64, false),
+        ]));
+
+        let mut buf = Vec::with_capacity(1024);
+        let mut s = StreamWriter::try_new(&mut buf, &schema).unwrap();
+        s.finish().unwrap();
+        drop(s);
+
+        let buffer = Buffer::from_vec(buf);
+
+        let mut b = buffer.slice_with_length(0, buffer.len() - 1);
+        let mut decoder = StreamDecoder::new();
+        let output = decoder.decode(&mut b).unwrap();
+        assert!(output.is_none());
+        let decoded_schema = decoder.schema().unwrap();
+        assert_eq!(schema, decoded_schema);
+
+        let err = decoder.finish().unwrap_err().to_string();
+        assert_eq!(err, "Ipc error: Unexpected End of Stream");
+    }
+
     #[test]
     fn test_read_ree_dict_record_batches_from_buffer() {
         let schema = Schema::new(vec![Field::new(
@@ -362,8 +395,7 @@ mod tests {
             let mut writer = StreamWriter::try_new_with_options(
                 &mut buffer,
                 &schema,
-                #[allow(deprecated)]
-                IpcWriteOptions::default().with_preserve_dict_id(false),
+                IpcWriteOptions::default(),
             )
             .expect("Failed to create StreamWriter");
             writer.write(&batch).expect("Failed to write RecordBatch");
@@ -375,7 +407,7 @@ mod tests {
         while let Some(batch) = decoder
             .decode(buf)
             .map_err(|e| {
-                ArrowError::ExternalError(format!("Failed to decode record batch: {}", e).into())
+                ArrowError::ExternalError(format!("Failed to decode record batch: {e}").into())
             })
             .expect("Failed to decode record batch")
         {
