@@ -33,6 +33,7 @@ use arrow_buffer::bit_util::ceil;
 use arrow_buffer::{BooleanBuffer, MutableBuffer, NullBuffer};
 use arrow_schema::ArrowError;
 use arrow_select::take::take;
+use std::cmp::Ordering;
 use std::ops::Not;
 
 #[derive(Debug, Copy, Clone)]
@@ -272,7 +273,7 @@ fn compare_op(op: Op, lhs: &dyn Datum, rhs: &dyn Datum) -> Result<BooleanArray,
                     let r = r.inner().bit_chunks().iter_padded();
                     let ne = values.bit_chunks().iter_padded();
 
-                    let c = |((l, r), n)| ((l ^ r) | (l & r & n));
+                    let c = |((l, r), n)| (l ^ r) | (l & r & n);
                     let buffer = l.zip(r).zip(ne).map(c).collect();
                     BooleanBuffer::new(buffer, 0, len).into()
                 }
@@ -565,24 +566,42 @@ impl<'a, T: ByteViewType> ArrayOrd for &'a GenericByteViewArray<T> {
     /// Item.0 is the array, Item.1 is the index
     type Item = (&'a GenericByteViewArray<T>, usize);
 
+    #[inline(always)]
     fn is_eq(l: Self::Item, r: Self::Item) -> bool {
-        // # Safety
-        // The index is within bounds as it is checked in value()
         let l_view = unsafe { l.0.views().get_unchecked(l.1) };
-        let l_len = *l_view as u32;
-
         let r_view = unsafe { r.0.views().get_unchecked(r.1) };
+        if l.0.data_buffers().is_empty() && r.0.data_buffers().is_empty() {
+            // For eq case, we can directly compare the inlined bytes
+            return l_view == r_view;
+        }
+
+        let l_len = *l_view as u32;
         let r_len = *r_view as u32;
         // This is a fast path for equality check.
         // We don't need to look at the actual bytes to determine if they are equal.
         if l_len != r_len {
             return false;
         }
+        if l_len == 0 && r_len == 0 {
+            return true;
+        }
 
+        // # Safety
+        // The index is within bounds as it is checked in value()
         unsafe { GenericByteViewArray::compare_unchecked(l.0, l.1, r.0, r.1).is_eq() }
     }
 
+    #[inline(always)]
     fn is_lt(l: Self::Item, r: Self::Item) -> bool {
+        // If both arrays use only the inline buffer
+        if l.0.data_buffers().is_empty() && r.0.data_buffers().is_empty() {
+            let l_view = unsafe { l.0.views().get_unchecked(l.1) };
+            let r_view = unsafe { r.0.views().get_unchecked(r.1) };
+            return GenericByteViewArray::<T>::inline_key_fast(*l_view)
+                < GenericByteViewArray::<T>::inline_key_fast(*r_view);
+        }
+
+        // Fallback to the generic, unchecked comparison for non-inline cases
         // # Safety
         // The index is within bounds as it is checked in value()
         unsafe { GenericByteViewArray::compare_unchecked(l.0, l.1, r.0, r.1).is_lt() }
@@ -618,78 +637,24 @@ impl<'a> ArrayOrd for &'a FixedSizeBinaryArray {
 }
 
 /// Compares two [`GenericByteViewArray`] at index `left_idx` and `right_idx`
+#[inline(always)]
 pub fn compare_byte_view<T: ByteViewType>(
     left: &GenericByteViewArray<T>,
     left_idx: usize,
     right: &GenericByteViewArray<T>,
     right_idx: usize,
-) -> std::cmp::Ordering {
+) -> Ordering {
     assert!(left_idx < left.len());
     assert!(right_idx < right.len());
+    if left.data_buffers().is_empty() && right.data_buffers().is_empty() {
+        let l_view = unsafe { left.views().get_unchecked(left_idx) };
+        let r_view = unsafe { right.views().get_unchecked(right_idx) };
+        return GenericByteViewArray::<T>::inline_key_fast(*l_view)
+            .cmp(&GenericByteViewArray::<T>::inline_key_fast(*r_view));
+    }
     unsafe { GenericByteViewArray::compare_unchecked(left, left_idx, right, right_idx) }
 }
 
-/// Comparing two [`GenericByteViewArray`] at index `left_idx` and `right_idx`
-///
-/// Comparing two ByteView types are non-trivial.
-/// It takes a bit of patience to understand why we don't just compare two &[u8] directly.
-///
-/// ByteView types give us the following two advantages, and we need to be careful not to lose them:
-/// (1) For string/byte smaller than 12 bytes, the entire data is inlined in the view.
-///     Meaning that reading one array element requires only one memory access
-///     (two memory access required for StringArray, one for offset buffer, the other for value buffer).
-///
-/// (2) For string/byte larger than 12 bytes, we can still be faster than (for certain operations) StringArray/ByteArray,
-///     thanks to the inlined 4 bytes.
-///     Consider equality check:
-///     If the first four bytes of the two strings are different, we can return false immediately (with just one memory access).
-///
-/// If we directly compare two &[u8], we materialize the entire string (i.e., make multiple memory accesses), which might be unnecessary.
-/// - Most of the time (eq, ord), we only need to look at the first 4 bytes to know the answer,
-///   e.g., if the inlined 4 bytes are different, we can directly return unequal without looking at the full string.
-///
-/// # Order check flow
-/// (1) if both string are smaller than 12 bytes, we can directly compare the data inlined to the view.
-/// (2) if any of the string is larger than 12 bytes, we need to compare the full string.
-///     (2.1) if the inlined 4 bytes are different, we can return the result immediately.
-///     (2.2) o.w., we need to compare the full string.
-///
-/// # Safety
-/// The left/right_idx must within range of each array
-#[deprecated(note = "Use `GenericByteViewArray::compare_unchecked` instead")]
-pub unsafe fn compare_byte_view_unchecked<T: ByteViewType>(
-    left: &GenericByteViewArray<T>,
-    left_idx: usize,
-    right: &GenericByteViewArray<T>,
-    right_idx: usize,
-) -> std::cmp::Ordering {
-    let l_view = left.views().get_unchecked(left_idx);
-    let l_len = *l_view as u32;
-
-    let r_view = right.views().get_unchecked(right_idx);
-    let r_len = *r_view as u32;
-
-    if l_len <= 12 && r_len <= 12 {
-        let l_data = unsafe { GenericByteViewArray::<T>::inline_value(l_view, l_len as usize) };
-        let r_data = unsafe { GenericByteViewArray::<T>::inline_value(r_view, r_len as usize) };
-        return l_data.cmp(r_data);
-    }
-
-    // one of the string is larger than 12 bytes,
-    // we then try to compare the inlined data first
-    let l_inlined_data = unsafe { GenericByteViewArray::<T>::inline_value(l_view, 4) };
-    let r_inlined_data = unsafe { GenericByteViewArray::<T>::inline_value(r_view, 4) };
-    if r_inlined_data != l_inlined_data {
-        return l_inlined_data.cmp(r_inlined_data);
-    }
-
-    // unfortunately, we need to compare the full data
-    let l_full_data: &[u8] = unsafe { left.value_unchecked(left_idx).as_ref() };
-    let r_full_data: &[u8] = unsafe { right.value_unchecked(right_idx).as_ref() };
-
-    l_full_data.cmp(r_full_data)
-}
-
 #[cfg(test)]
 mod tests {
     use std::sync::Arc;
