@@ -27,13 +27,14 @@ use arrow::datatypes::{Field, Float32Type, Int32Type, Int64Type, Schema, UInt8Ty
 
 use arrow_array::types::Decimal128Type;
 use criterion::{criterion_group, criterion_main, Criterion};
+use std::hint;
 
 fn bench_filter(data_array: &dyn Array, filter_array: &BooleanArray) {
-    criterion::black_box(filter(data_array, filter_array).unwrap());
+    hint::black_box(filter(data_array, filter_array).unwrap());
 }
 
 fn bench_built_filter(filter: &FilterPredicate, array: &dyn Array) {
-    criterion::black_box(filter.filter(array).unwrap());
+    hint::black_box(filter.filter(array).unwrap());
 }
 
 fn add_benchmark(c: &mut Criterion) {
