@@ -18,7 +18,7 @@
 //! Module for parsing JSON strings as Variant
 
 use arrow_schema::ArrowError;
-use parquet_variant::{ListBuilder, ObjectBuilder, Variant, VariantBuilderExt};
+use parquet_variant::{ObjectFieldBuilder, Variant, VariantBuilderExt};
 use serde_json::{Number, Value};
 
 /// Converts a JSON string to Variant using a [`VariantBuilderExt`], such as
@@ -120,37 +120,15 @@ fn append_json(json: &Value, builder: &mut impl VariantBuilderExt) -> Result<(),
         Value::Object(obj) => {
             let mut obj_builder = builder.try_new_object()?;
             for (key, value) in obj.iter() {
-                let mut field_builder = ObjectFieldBuilder {
-                    key,
-                    builder: &mut obj_builder,
-                };
+                let mut field_builder = ObjectFieldBuilder::new(key, &mut obj_builder);
                 append_json(value, &mut field_builder)?;
             }
-            obj_builder.finish()?;
+            obj_builder.finish();
         }
     };
     Ok(())
 }
 
-struct ObjectFieldBuilder<'o, 'v, 's> {
-    key: &'s str,
-    builder: &'o mut ObjectBuilder<'v>,
-}
-
-impl VariantBuilderExt for ObjectFieldBuilder<'_, '_, '_> {
-    fn append_value<'m, 'v>(&mut self, value: impl Into<Variant<'m, 'v>>) {
-        self.builder.insert(self.key, value);
-    }
-
-    fn try_new_list(&mut self) -> Result<ListBuilder<'_>, ArrowError> {
-        self.builder.try_new_list(self.key)
-    }
-
-    fn try_new_object(&mut self) -> Result<ObjectBuilder<'_>, ArrowError> {
-        self.builder.try_new_object(self.key)
-    }
-}
-
 #[cfg(test)]
 mod test {
     use super::*;
@@ -489,7 +467,7 @@ mod test {
         let mut list_builder = variant_builder.new_list();
         let mut object_builder_inner = list_builder.new_object();
         object_builder_inner.insert("age", Variant::Int8(32));
-        object_builder_inner.finish().unwrap();
+        object_builder_inner.finish();
         list_builder.append_value(Variant::Int16(128));
         list_builder.append_value(Variant::BooleanFalse);
         list_builder.finish();
@@ -553,7 +531,7 @@ mod test {
         let mut object_builder = variant_builder.new_object();
         object_builder.insert("a", Variant::Int8(3));
         object_builder.insert("b", Variant::Int8(2));
-        object_builder.finish().unwrap();
+        object_builder.finish();
         let (metadata, value) = variant_builder.finish();
         let variant = Variant::try_new(&metadata, &value)?;
         JsonToVariantTest {
@@ -577,7 +555,7 @@ mod test {
         inner_list_builder.append_value(Variant::Double(-3e0));
         inner_list_builder.append_value(Variant::Double(1001e-3));
         inner_list_builder.finish();
-        object_builder.finish().unwrap();
+        object_builder.finish();
         let (metadata, value) = variant_builder.finish();
         let variant = Variant::try_new(&metadata, &value)?;
         JsonToVariantTest {
@@ -643,9 +621,9 @@ mod test {
                 }
                 list_builder.finish();
             });
-            inner_object_builder.finish().unwrap();
+            inner_object_builder.finish();
         });
-        object_builder.finish().unwrap();
+        object_builder.finish();
         let (metadata, value) = variant_builder.finish();
         let variant = Variant::try_new(&metadata, &value)?;
 
@@ -669,7 +647,7 @@ mod test {
         let mut object_builder = variant_builder.new_object();
         object_builder.insert("a", Variant::Int8(1));
         object_builder.insert("爱", Variant::ShortString(ShortString::try_new("अ")?));
-        object_builder.finish().unwrap();
+        object_builder.finish();
         let (metadata, value) = variant_builder.finish();
         let variant = Variant::try_new(&metadata, &value)?;
 
