@@ -77,7 +77,7 @@ fn bench_object_field_names_reverse_order(c: &mut Criterion) {
                 object_builder.insert(format!("{}", 1000 - i).as_str(), string_table.next());
             }
 
-            object_builder.finish().unwrap();
+            object_builder.finish();
             hint::black_box(variant.finish());
         })
     });
@@ -113,7 +113,7 @@ fn bench_object_same_schema(c: &mut Criterion) {
                 inner_list_builder.append_value(string_table.next());
 
                 inner_list_builder.finish();
-                object_builder.finish().unwrap();
+                object_builder.finish();
 
                 hint::black_box(variant.finish());
             }
@@ -154,7 +154,7 @@ fn bench_object_list_same_schema(c: &mut Criterion) {
                 list_builder.append_value(string_table.next());
 
                 list_builder.finish();
-                object_builder.finish().unwrap();
+                object_builder.finish();
             }
 
             list_builder.finish();
@@ -189,7 +189,7 @@ fn bench_object_unknown_schema(c: &mut Criterion) {
                             let key = string_table.next();
                             inner_object_builder.insert(key, key);
                         }
-                        inner_object_builder.finish().unwrap();
+                        inner_object_builder.finish();
 
                         continue;
                     }
@@ -202,7 +202,7 @@ fn bench_object_unknown_schema(c: &mut Criterion) {
 
                     inner_list_builder.finish();
                 }
-                object_builder.finish().unwrap();
+                object_builder.finish();
                 hint::black_box(variant.finish());
             }
         })
@@ -241,7 +241,7 @@ fn bench_object_list_unknown_schema(c: &mut Criterion) {
                             let key = string_table.next();
                             inner_object_builder.insert(key, key);
                         }
-                        inner_object_builder.finish().unwrap();
+                        inner_object_builder.finish();
 
                         continue;
                     }
@@ -254,7 +254,7 @@ fn bench_object_list_unknown_schema(c: &mut Criterion) {
 
                     inner_list_builder.finish();
                 }
-                object_builder.finish().unwrap();
+                object_builder.finish();
             }
 
             list_builder.finish();
@@ -314,10 +314,10 @@ fn bench_object_partially_same_schema(c: &mut Criterion) {
                         let key = string_table.next();
                         inner_object_builder.insert(key, key);
                     }
-                    inner_object_builder.finish().unwrap();
+                    inner_object_builder.finish();
                 }
 
-                object_builder.finish().unwrap();
+                object_builder.finish();
                 hint::black_box(variant.finish());
             }
         })
@@ -376,10 +376,10 @@ fn bench_object_list_partially_same_schema(c: &mut Criterion) {
                         let key = string_table.next();
                         inner_object_builder.insert(key, key);
                     }
-                    inner_object_builder.finish().unwrap();
+                    inner_object_builder.finish();
                 }
 
-                object_builder.finish().unwrap();
+                object_builder.finish();
             }
 
             list_builder.finish();
@@ -408,7 +408,7 @@ fn bench_validation_validated_vs_unvalidated(c: &mut Criterion) {
         }
         list.finish();
 
-        obj.finish().unwrap();
+        obj.finish();
         test_data.push(builder.finish());
     }
 
@@ -462,7 +462,7 @@ fn bench_iteration_performance(c: &mut Criterion) {
         let mut obj = list.new_object();
         obj.insert(&format!("field_{i}"), rng.random::<i32>());
         obj.insert("nested_data", format!("data_{i}").as_str());
-        obj.finish().unwrap();
+        obj.finish();
     }
     list.finish();
 
